#!/usr/bin/env python3
#
# Capture some statististics about a single umaudemc execution using the Python profiler
#

import cProfile
import csv
import os
import pstats
import runpy
import shlex
import sys

# The first argument is the name of the CSV file to write the results
csv_file = sys.argv[1]

# All other arguments are forwarded to umaudemc
profile = cProfile.Profile()
sys.argv[1:] = sys.argv[2:]

# Disable sys.exit to recover the control flow
sys.exit = lambda _: None

# Disable StormPy if requested
if os.getenv('NO_STORMPY'):
	import importlib
	find_spec = importlib.util.find_spec
	importlib.util.find_spec = lambda name: None if name == 'stormpy' else find_spec(name)

profile.enable()
runpy.run_module('umaudemc', run_name='__main__', alter_sys=False)
profile.disable()

# Get the statististics and write some values into the CSV
stats = pstats.Stats(profile, stream=sys.stdout)
stats = stats.get_stats_profile()

def get_cumtime(stat, func):
	"""Get accumulated time for a function"""
	profile = stats.func_profiles.get(func)
	return profile.cumtime if profile else 0.0

with open(csv_file, 'a') as csvfile:
	csvw = csv.writer(csvfile)

	prism_path = os.getenv('PRISM_PATH')

	csvw.writerow([
		shlex.join(sys.argv[1:]),
		prism_path is not None and 'ngprism' in prism_path,
		os.getenv('NO_STORMPY') is not None,
		stats.total_tt,
		stats.func_profiles['pcheck'].cumtime,
		get_cumtime(stats, 'run'),
		stats.func_profiles['get_probabilistic_graph'].cumtime,
		stats.func_profiles['parse_initial_data'].cumtime,
		stats.func_profiles['init'].cumtime,
		stats.func_profiles['load'].cumtime,
		get_cumtime(stats, 'graph'),
		get_cumtime(stats, '_build_model'),
		get_cumtime(stats, 'model_checking'),
	])
